static char rcsid[] = "@(#)$Id: answer.c,v 2.3 2009-10-21 04:22:19 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.3 $   $State: Exp $
 *
 *  Modified by: Kari Hurtta <hurtta+elm@posti.FMI.FI> 
 *                       (was hurtta+elm@ozone.FMI.FI)
 *           or  Kari Hurtta <elm@elmme-mailer.org>
 ******************************************************************************
 *  Based on Elm 2.4 utils/answer.c. That code was following copyright:
 *
 *  The Elm Mail System 
 *
 * 			Copyright (c) 1988-1992 USENET Community Trust
 * 			Copyright (c) 1986,1987 Dave Taylor
 *****************************************************************************/

/** This program is a phone message transcription system, and
    is designed for secretaries and the like, to allow them to
    painlessly generate electronic mail instead of paper forms.

    Note: this program ONLY uses the local alias file, and does not
	  even read in the system alias file at all.

**/
#include "def_utils.h"
#include "s_answer.h"
#include "sysdefs.h"

#include "addrlib.h"
#include "aliaslib.h"

DEBUG_VAR(Debug,__FILE__,"util");

#define  ELM		"elm"		/* where the elm program lives */

static int translate P_((char *fullname, char *name, int namesize));


static char *quit_word, *exit_word, *done_word, *bye_word;

static char version_buff[NLEN];


static unsigned char * s2us P_((char *str));
static unsigned char * s2us(str)
     char *str;
{
    return (unsigned char *)str;
}



#if ANSI_C
alias_stack_lookup_f lookup_dummy;
#endif
int lookup_dummy(value,alias,result_alias,result_list)
     union alias_stack_item     value;
     const struct string       *alias;
     const struct address_alias **result_alias;
     /* Caller must free result_list */
     struct addr_list          **result_list;
{
    /* Only ASCII accounts are accepted on here ... */

    if (!can_ascii_string(alias)) {
	lib_error(CATGETS(elm_msg_cat, AnswerSet,
			  AnswerAsciiRequiredAddress,
			  "Ascii required for address %S"),
		  alias);
	return 0;
    }


    {
	struct address * address =  new_address_string (alias,NULL,NULL);

	*result_list = new_empty_addr_list(1);
	
	add_address_to_list(*result_list,address,-1);
       	
    }

    return 1;

}



int main P_((int argc, char *argv[]));
int main(argc, argv)
     int argc;
     char *argv[];
{
	FILE *fd;
	char buffer[LONG_STRING], tempfile[SLEN], *cp;
	char   in_line[SLEN];
	int   msgnum = 0, eof, allow_name = 0, phone_slip = 0;
	int   ans_pid = getpid();
	int c;
	int i;

	extern int optind;
	extern char *optarg;

	struct alias_stack * stack = NULL;    

#if DEBUG	
	init_debugfile("ANSWER");
#endif
	locale_init();

	quit_word = catgets(elm_msg_cat, AnswerSet, AnswerQuitWord, "quit");
	exit_word = catgets(elm_msg_cat, AnswerSet, AnswerExitWord, "exit");
	done_word = catgets(elm_msg_cat, AnswerSet, AnswerDoneWord, "done");
	bye_word = catgets(elm_msg_cat, AnswerSet, AnswerByeWord, "bye");

	user_init();
	init_aliaslib();
	init_defaults();

/*
 *	-u = allow user names not in alias table
 *	-p = prompt for phone slip messages
 */
	while ((c = getopt(argc, argv, "upd:r:")) != EOF) {
	    switch (c) {
	    case 'u':
		allow_name = 1;
		break;
	    case 'p':
		phone_slip = 1;
		break;
	    case 'd':
#if DEBUG
		set_debugging(optarg);	  
#endif
		break;
	    case 'r':
		set_user_rc_location(optarg); 
		break;
	  }
	}
	
	read_rc_file(0);

	stack = new_alias_stack();

	if (user_aliases_map) {
	    union alias_stack_item map_info;

	    map_info.alias_map = user_aliases_map;

	    add_lookup_to_alias_stack(stack,alias_simple_lookup,map_info);
	}


	if (system_aliases_map) {
	    union alias_stack_item map_info;

	    map_info.alias_map = system_aliases_map;

	    add_lookup_to_alias_stack(stack,alias_simple_lookup,map_info);
	}

	if (allow_name) {
	    union alias_stack_item  mailer;
	    mailer.dummy_arg = NULL;
		
	    add_lookup_to_alias_stack(stack,lookup_dummy,mailer);
	}
	    


	elm_sfprintf(version_buff, sizeof version_buff,
		     FRM("%s PL%s"), VERSION, PATCHLEVEL);


#ifdef DEBUG
    { 
	int d = panic_dprint("\n\
======================================================\n\
Debug output of the ANSWER program (version %s).\n",
			     version_buff);

	if (d >= 50) {
#if 0	
	    panic_dprint("WARNING: Edit manually out sensitive information from that file!\n");
    
	    lower_prompt("WARNING: Debug file may include passwords -- edit it!");
	    sleep(5+sleepmsg);	    
#endif
	}
    }
#endif

    while (1) {
	struct addr_list  * addr_list = NULL;
	char *              list_text = NULL;
	char name[SLEN], user_name[SLEN];

	struct string * name_x = NULL;
	int count;

	  if (msgnum > 9999) msgnum = 0;
	
	  printf("\n-------------------------------------------------------------------------------\n");

prompt:   elm_fprintf(stdout,
		      CATGETS(elm_msg_cat, AnswerSet, AnswerMessageTo, 
			      "\nMessage to: "));
	  if (fgets(user_name, SLEN, stdin) == NULL) {
		putchar('\n');
		exit(0);
	  }
	  if(user_name[0] == '\0')
	    goto prompt;
	  
	  cp = &user_name[strlen(user_name)-1];

	  if(*cp == '\n') *cp = '\0';
	  if(user_name[0] == '\0')
		goto prompt;

	  if ((istrcmp(user_name, quit_word) == 0) ||
	      (istrcmp(user_name, exit_word) == 0) ||
	      (istrcmp(user_name, done_word) == 0) ||
	      (istrcmp(user_name, bye_word)  == 0))
	     exit(0);

	  if (translate(user_name, name, sizeof name) == 0)
	      goto prompt;


	  name_x = new_string2(system_charset,s2us(name));

	  addr_list = aliases_expand_alias(stack,name_x);

	  free_string(&name_x);

	  if (!addr_list) {
	      lib_error(CATGETS(elm_msg_cat, AnswerSet, AnswerSorryNotFound,
				"Sorry, could not find '%s' [%s] in list!\n"),
			user_name, name);
	      goto prompt;
	  }

	  count = addr_list_item_count(addr_list);
	  if (!count)
	      goto no_addr;

	  list_text = list_to_str(addr_list,0,system_charset);
	  printf("address '%s'\n", list_text);
	  free(list_text);
	  
	  elm_sfprintf(tempfile, sizeof tempfile,
		       FRM("%sans.%d.%d"), 
		       default_temp, ans_pid, msgnum++);

	  if ((fd = fopen(tempfile,"w+")) == NULL) {
	    lib_error(CATGETS(elm_msg_cat, AnswerSet, AnswerCouldNotOpenWrite,
			      "** Fatal Error: could not open %s to write\n"),
		      tempfile);
	    exit(1);
	  }

	/** Enter standard phone message fields **/
	  if (phone_slip) {
	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, AnswerCaller, 
				    "Caller: "),
		    sizeof buffer);
	    printf("\n%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, AnswerOf, 
				    "of:     "),
		    sizeof buffer);
	    printf("%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, AnswerPhone, 
				    "Phone:  "),
		    sizeof buffer);
	    printf("%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s\n",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, AnswerTelephoned, 
				    "TELEPHONED         - "),
		    sizeof buffer);
	    printf("\n%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, 
				    AnswerCalledToSeeYou, 
				    "CALLED TO SEE YOU  - "),
		    sizeof buffer);
	    printf("%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, 
				    AnswerWantsToSeeYou, 
				    "WANTS TO SEE YOU   - "),
		    sizeof buffer);
	    printf("%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, 
				    AnswerReturnedYourCall, 
				    "RETURNED YOUR CALL - "),
		    sizeof buffer);
	    printf("%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, AnswerPleaseCall, 
				    "PLEASE CALL        - "),
		    sizeof buffer);
	    printf("%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, 
				    AnswerWillCallAgain, 
				    "WILL CALL AGAIN    - "),
		    sizeof buffer);
	    printf("%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);

	    strfcpy(buffer, catgets(elm_msg_cat, AnswerSet, AnswerUrgent, 
				    "*****URGENT******  - "),
		    sizeof buffer);
	    printf("%s",buffer);
	    fflush(stdout);
	    fgets(in_line, SLEN, stdin);
	    if (strlen(in_line) > 1)
	      fprintf(fd,"%s%s",buffer,in_line);
	  }

	  elm_fprintf(stdout,
		      CATGETS(elm_msg_cat, AnswerSet, AnswerEnterMessage,
			      "\n\nEnter message for %s ending with a blank line.\n\n"), 
		      user_name);

	  fprintf(fd,"\n\n");

	  do {
	   printf("> ");
	   if (! (eof = (fgets(buffer, SLEN, stdin) == NULL))) 
	     fprintf(fd, "%s", buffer);
	  } while (! eof && strlen(buffer) > 1);
	
	  rewind(fd);
#ifdef _POSIX_VERSION
	  /* Synzronize underlying file descriptor */
	  fflush(fd);
#else
	  seek(fileno(fd),0,0);
#endif


	  {
	      int ret;
	      int len = count + 5;
	      struct run_state RS;
	      int x = 0,i,j;

	      char **   addrs = safe_malloc(sizeof(char *) * count);
	      const char ** argv = safe_malloc(sizeof(char *) * len);

	      /* XXXX --- not correct? */

	      char * subject = elm_message(CATGETS(elm_msg_cat, AnswerSet, 
						   AnswerWhileOut,
						   "While You Were Out"));


	      argv[x++] = ELM;
	      argv[x++] = "-s";
	      argv[x++] = subject;
	      argv[x++] = "--";

	      for (i = 0; i < count && x < len-1; i++) {
		  int group = -1;
		  const struct address * addr = 
		      addr_list_get_item(addr_list,i,
					 &group);

		  char * str =
		      address_to_str(addr,
				     /* XXX is this correct? */
				     !allow_no_hdrencoding,
				     system_charset);

		  
		  addrs[i] = str;
		  argv[x++] = addrs[i];
	      }
	      argv[x] = NULL;


	      ret = start_run(&RS,0,argv,fileno(fd),-1);
	      
	      if (ret) {
		  int exit_code;


		  /* XXXX -- add background */

		  ret = wait_end(&RS,&exit_code);

		  if (0 == ret) {
		      DPRINT(Debug,1,(&Debug,
				      "Exit status of %s lost\n",
				      argv[0]));

		  } else if (ret < 0) {
		      lib_error(CATGETS(elm_msg_cat, AnswerSet,	AnswerProgDies,
					"%s died on signal %d"),
			    argv[0],-ret);
		      
		  } else if (exit_code) {
		      lib_error(CATGETS(elm_msg_cat, AnswerSet,	AnswerProgExit,
					"%s exited with status %d"),
				argv[0],exit_code);
		      
		  }
		  
	      } 

	      if (0 == ret) {
		  if (RS.save_errno)		      
		      lib_error(CATGETS(elm_msg_cat, AnswerSet,	AnswerProgFailed,
					"Failed to run %s: %s"),
				argv[0],strerror(RS.save_errno));
		  else
		      lib_error(CATGETS(elm_msg_cat, AnswerSet,	AnswerProgFailed0,
					"Failed to run %s"),
				argv[0]);
		  		  
	      }
	      
	      
	      for (j = 0; j < i; j++) {
		  free(addrs[j]);
		  addrs[j] = NULL;
	      }

	      free(addrs);
	      free(argv);
	      free(subject);

	  }

	  fclose(fd);

    no_addr:
	  free_addr_list(&addr_list);
	}
}

static int translate(fullname, name, namesize)
     char *fullname, *name;
     int namesize;
{
	/** translate fullname into name..
	       'first last'  translated to first_initial - underline - last
	       'initial last' translated to initial - underline - last
	    Return 0 if error.
	**/
	register int i, lastname = 0, len;

	for (i=0, len = strlen(fullname); i < len; i++) {

#ifdef ASCII_CTYPE
	  if (isascii(fullname[i]))
#endif
	    fullname[i] = tolower(fullname[i]);

	  if (fullname[i] == ' ') {
	      if (lastname) {
		  lib_error(CATGETS(elm_msg_cat, AnswerSet, AnswerCannotHaveMoreNames,
				    "** Can't have more than 'FirstName LastName' as address!\n"));
		  return(0);
	      }  else
		  lastname = i+1;
	  }
	}

	if (lastname) 
	  elm_sfprintf(name, namesize,
		       FRM("%c_%.*s"), 
		       fullname[0], namesize-4, (char *) fullname + lastname);
	else
	  strfcpy(name, fullname, namesize);

	return(1);
}


/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
